/*
 * $Id: Command.java,v 1.3 2002-09-01 15:08:40-07 ball Exp $
 *
 * Copyright 2002 Allen D. Ball.  All rights reserved.
 */
package player;

import world.*;

/**
 * Abstract class that implements robot commmands to send to the game
 * server.
 *
 * @author <a href="mailto:ball@iprotium.com">Allen D. Ball</a>
 * @version $Revision: 1.3 $
 */
public abstract class Command {
    private int bid = 1;

    /**
     * Creates a robot command.
     */
    protected Command() { }

    /**
     * Method to get the bid for this command.
     *
     * @return                  The bid for this command.
     */
    public final int getBid() { return bid; }

    /**
     * Method to set the bid for this command.
     *
     * @param   bid             The bid for the command.
     */
    public final void setBid(int bid) {
        if (bid == 0) {
            throw new IllegalArgumentException("Bid cannot equal zero.");
        }

        this.bid = bid;
    }

    /**
     * Method to get the cost of this command.  The cost is equal to
     * the absolute value of the bid.
     *
     * @return                  The cost of this command.
     */
    public final int getCost() {
        int cost = bid;

        if (cost < 0) {
            cost = - cost;
        }

        return cost;
    }

    /**
     * Method to get the name of this command.
     *
     * @return                  The name of this command.
     */
    public abstract String getName();

    /**
     * Method to set the movement direction of this command.
     * (Method defined in {@link Command} is a no-op.  Subclasses
     * should override this method.)
     *
     * @param   direction       The direction of the movement.
     *
     * @see Direction
     */
    public void setDirection(int direction) { }

    /**
     * Method to add a package to the command package list.
     * (Method defined in {@link Command} is a no-op.  Subclasses
     * should override this method.)
     *
     * @param   pkg             The package to add to the command list.
     */
    public void pkgAdd(Pkg pkg) { }

    /**
     * Method to remove a package to the command package list.
     * (Method defined in {@link Command} is a no-op.  Subclasses
     * should override this method.)
     *
     * @param   pkg             The package to remove from the command list.
     */
    public void pkgRemove(Pkg pkg) { }

    /**
     * Method to get the optional arguments for a command
     *
     * @return                  The optional arguments for this commnd.
     */
    public abstract String getArguments();

    /**
     * Method to return the command as a well formatted string suitable
     * for sending to the game server.
     *
     * @return                  The command string to send to the server.
     */
    public final String toString() {
        return Integer.toString(getBid())
               + " " + getName() + " " + getArguments();
    }
}
/*
 * $Log: Command.java,v $
 * Revision 1.3  2002-09-01 15:08:40-07  ball
 * Made setting of command arguments consistent for all subclasses
 * of Command.
 * Added an initialization method to Strategy classes.
 * Player will submit a no-op move if the Strategy implementation
 * does not return a command.
 *
 * Revision 1.2  2002-09-01 08:38:12-07  ball
 * Properly handle direction and package arguments to server commands.
 * Move Strategy class implementations to their own package.
 * Make board two rows higher and two columns wider than the server
 * specifies.  No longer translate one-based server coordinates to
 * zero-based co-ordinates.  Make the unspecified tiles on the edges wall
 * tiles.  This removes all special handling logic because all edge tiles
 * are guaranteed to be wall tiles (and, therefore, impassable).
 *
 * Revision 1.1  2002-08-31 17:46:24-07  ball
 * Initial revision
 *
 */
