/*
 * $Id: Main.java,v 1.3 2002-09-02 10:09:07-07 ball Exp $
 *
 * Copyright 2002 Allen D. Ball.  All rights reserved.
 */
package player;

import gui.*;
import java.io.*;
import java.net.*;
import javax.swing.*;
import player.*;
import protocol.*;
import world.*;

/**
 * Provides the main() function for the robot, a ICFP-2002 programming
 * contest entry.
 *
 * @author <a href="mailto:ball@iprotium.com">Allen D. Ball</a>
 * @version $Revision: 1.3 $
 */
public class Main extends Thread {
    private static final String title = "Robot Player";
    private static boolean useGUI = Boolean.getBoolean("useGUI");
    private static boolean trace = useGUI ? true : Boolean.getBoolean("trace");
    private static final String macOSUseScreenMenuBarProperty =
        "com.apple.macos.useScreenMenuBar";

    /**
     * Static main() function for the robot application.  Creates an
     * instance of Main and calls the start() method.
     *
     * @param   argv    Array of strings representing the command line
     *                  arguments.
     */
    public static void main(String[] argv) {
        System.setProperty(macOSUseScreenMenuBarProperty, "true");

        new Main(argv).start();
    }

    private String[] argv = new String[0];
    private String server = "localhost";
    private int port = 0;
    private World world = new World();
    private Player player = new Player(world);
    private GUI gui = null;

    private Main(String[] argv) {
        super(title);
        this.argv = argv;
    }

    private Main() { this(new String[0]); }

    /**
     * Does the work of setting up and starting the player.
     */
    public synchronized void run() {
        PrintStream tracePrintStream = null;

        /* If running as a GUI, Re-attach the System in, out, and err
         * streams to those of the GUI's tty.
         */
        if (useGUI) {
            GUI gui = new GUI("Monitor", world, player);

            System.setIn(gui.getTTY().getIn());
            System.setOut(gui.getTTY().getOut());
            System.setErr(gui.getTTY().getErr());

            gui.pack();
            gui.setVisible(true);

            player.setGUI(gui);

            Thread.currentThread().yield();
        }

        /* If trace output is desired, set the tracePrintStream to a
         * non-null value to direct the output.
         */
        if (trace) {
            tracePrintStream = System.out;
        }

        switch (argv.length) {
        case 2:
            server = argv[0];
            port = Integer.parseInt(argv[1]);
            break;

        default:
            throw new IllegalArgumentException("Invalid command line.");
            // break;
        }

        try {
            Socket socket = new Socket(server, port);
            player.setConnection(new Connection(socket, tracePrintStream));
            player.start();
        } catch (IOException e) {
            System.err.println("Cannot connect to server "
                               + server + ":" + port);
            e.printStackTrace();
        }
    }
}
/*
 * $Log: Main.java,v $
 * Revision 1.3  2002-09-02 10:09:07-07  ball
 * Tried to improve GUI behavior on MacOS with a yield().
 * Improved accounting for dead robots.
 * Improved generation of board.
 *
 * Revision 1.2  2002-09-01 15:08:40-07  ball
 * Made setting of command arguments consistent for all subclasses
 * of Command.
 * Added an initialization method to Strategy classes.
 * Player will submit a no-op move if the Strategy implementation
 * does not return a command.
 *
 * Revision 1.1  2002-08-31 17:46:24-07  ball
 * Initial revision
 *
 */
