/*
 * $Id: Player.java,v 1.6 2002-09-02 10:09:07-07 ball Exp $
 *
 * Copyright 2002 Allen D. Ball.  All rights reserved.
 */
package player;

import gui.*;
import java.io.*;
import java.util.*;
import player.strategy.*;
import protocol.*;
import world.*;

/**
 * Class that implements a player in the ICFP-2002 programming contest.
 *
 * @author <a href="mailto:ball@iprotium.com">Allen D. Ball</a>
 * @version $Revision: 1.6 $
 */
public class Player extends Thread {
    static private Command noop = new PickCommand();

    private World world = null;
    private Connection connection = null;
    private GUI gui = null;
    private PlayerRobot robot = null;
    private Strategy strategy = new PlanA();

    /**
     * Creates a player thread.
     *
     * @param world             The world environment of the game.
     */
    public Player(World world) {
        super("Player");
        this.world = world;
    }

    private Player() { }

    /**
     * Method to get the player's world.
     *
     * @return                  The player's world.
     */
    public synchronized World getWorld() { return world; }

    /**
     * Method to get the player's connection.
     *
     * @return                  The connection connecting to the game
     *                          server.
     */
    public synchronized Connection getConnection() { return connection; }

    /**
     * Method to set the player's connection.
     *
     * @param   connection      The connection connecting to the game
     *                          server.
     */
    public synchronized void setConnection(Connection connection) {
        this.connection = connection;
    }

    /**
     * Method to set the GUI interface.  The GUI is notified at end of
     * each turn.
     *
     * @param   gui             The GUI.
     */
    public synchronized void setGUI(GUI gui) { this.gui = gui; }

    /**
     * Method to get the player's robot.
     *
     * @return                  The player's robot.
     */
    public synchronized PlayerRobot getPlayerRobot() { return robot; }

    public synchronized void run() {
        try {
            /* Initialize the connection.
             */
            connection.send("Player");

            /* Read and create the board.
             */
            readBoardConfiguration(connection.read());

            /* Read our robot's details.
             */
            readPlayerConfiguration(connection.read());

            /* Read the initial robot updates.
             */
            readRobotUpdates(connection.read());

            /* Initialize the player's strategy.
             */
            strategy.initialize(this);

            /* Notify GUI set-up is complete.
             */
            if (gui != null) {
                gui.notifyTurn();
            }

            while (world.getRobotMap().containsValue(robot)) {
                world.turn();

                /* Read the list of packages available at the player's
                 * robot's location.
                 */
                readPkgsAtCurrentPosition(connection.read());

                /* Consult our strategy and submit our move.
                 */
                Command command = strategy.nextCommand();
                if (command == null) {
                    System.err.println("Turn "+ world.getTurn() + ": "
                                       + strategy.getClass().getName()
                                       +" returned no command!");
                    command = noop;
                }

                connection.send(command);
                robot.setMoney(robot.getMoney() - command.getCost());

                /* Read the robot updates.
                 * (Skip over anything that does not start with "#" to
                 * skip server messages at end of conversation).
                 */
                String line = connection.read();
                while (line == null
                       || line.charAt(0) != '#') {
                    System.out.println(line != null ? line : "(null)");
                    line = connection.read();
                }

                readRobotUpdates(line);

                /* Notify GUI turn is complete.
                 */
                if (gui != null) {
                    gui.notifyTurn();
                }
            }

            System.out.println("Our robot died.");
        } catch (EOFException e) {
            System.out.println("EOF encountered communicating with server.");
        } catch (IOException e) {
            System.err.println("I/O exception communicating with server.");
            e.printStackTrace();
        }
    }

    private void readBoardConfiguration(String line) throws IOException {
        String[] tokens = getTokens(line);
        Board board =
            new Board(Integer.parseInt(tokens[1]) + 2,
                      Integer.parseInt(tokens[0]) + 2);

        world.setBoard(board);

        int i;
        int j;

        i = 0;
        for (j = 0; j < board.getColumnCount(); j += 1) {
            board.setTile(new WallTile(board, i, j));
        }

        for (i = 1; i < board.getRowCount() - 1; i += 1) {
            line = connection.read();

            j = 0;
            board.setTile(new WallTile(board, i, j));

            for (j = 1; j < board.getColumnCount() - 1; j += 1) {
                switch (line.charAt(j - 1)) {
                case '.':
                default:
                    board.setTile(new PlainTile(board, i, j));
                    break;

                case '~':
                    board.setTile(new WaterTile(board, i, j));
                    break;

                case '#':
                    board.setTile(new WallTile(board, i, j));
                    break;

                case '@':
                    board.setTile(new BaseTile(board, i, j));
                    break;
                }
            }

            j = board.getColumnCount() - 1;
            board.setTile(new WallTile(board, i, j));
        }

        i = board.getRowCount() - 1;
        for (j = 0; j < board.getColumnCount(); j += 1) {
            board.setTile(new WallTile(board, i, j));
        }
    }

    private void readPlayerConfiguration(String line) throws IOException {
        String[] tokens = getTokens(line);

        robot =
            new PlayerRobot(world,
                            Integer.parseInt(tokens[0]),
                            Integer.parseInt(tokens[1]),
                            Integer.parseInt(tokens[2]));
        world.getRobotMap().put(robot);
    }

    private void readPkgsAtCurrentPosition(String line) {
        String[] tokens = getTokens(line);
        Board board = world.getBoard();

        robot.getTile().getSack().empty();

        /* Package specification: id destination-column destination-row weight
         */
        for (int i = 0; (i + 4) <= tokens.length; i += 4) {
            Pkg pkg =
                new Pkg(Integer.parseInt(tokens[i + 0]),
                        Integer.parseInt(tokens[i + 3]),
                        board.getTile(Integer.parseInt(tokens[i + 2]),
                                      Integer.parseInt(tokens[i + 1])));
            robot.getTile().getSack().put(pkg);
        }
    }

    private void readRobotUpdates(String line) {
        String[] tokens = getTokens(line);
        Board board = world.getBoard();
        RobotMap robotMap = world.getRobotMap();
        RobotMap living = new RobotMap();

        Robot robot = null;
        int id = -1;
        int x = -1;

        for (int i = 0; i < tokens.length; i += 1) {
            switch (tokens[i].charAt(0)) {
            case '#':
                id = Integer.parseInt(tokens[i].substring(1));
                robot = (Robot) robotMap.get(id);
                if (robot == null) {
                    robot = new OpponentRobot(world, id);
                    robotMap.put(robot);
                }
                living.put(robot);
                break;

            case 'N':
                robot.moveNorth();
                break;

            case 'S':
                robot.moveSouth();
                break;

            case 'E':
                robot.moveEast();
                break;

            case 'W':
                robot.moveWest();
                break;

            case 'P':
                robot.pick(Integer.parseInt(tokens[++i]));
                break;

            case 'D':
                robot.drop(Integer.parseInt(tokens[++i]));
                break;

            case 'X':
                /*
                 * The protocol was changed to guarantee a "Y" command
                 * would always follow a "X" command.  Simply save the
                 * value for when we get the "Y" and process it there.
                 */
                x = Integer.parseInt(tokens[++i]);
                break;

            case 'Y':
                robot.setTile(board.getTile(Integer.parseInt(tokens[++i]), x));
                break;
            }
        }

        /* Keep the living.
         */
        robotMap.values().retainAll(living.values());
    }

    private static String[] getTokens(String line) {
        String string = line.trim();
        List tokenList = new ArrayList();

        while (string != null && (! string.equals(""))) {
            int spacePosition = string.indexOf(" ");

            if (spacePosition > 0) {
                tokenList.add(string.substring(0, spacePosition).trim());
                string = string.substring(spacePosition).trim();
            } else if (spacePosition < 0) {
                tokenList.add(string);
                string = null;
            } else /* if (spacePosition == 0) */ {
                string = string.trim();
            }
        }

        return (String[]) tokenList.toArray(new String[0]);
    }
}
/*
 * $Log: Player.java,v $
 * Revision 1.6  2002-09-02 10:09:07-07  ball
 * Tried to improve GUI behavior on MacOS with a yield().
 * Improved accounting for dead robots.
 * Improved generation of board.
 *
 * Revision 1.5  2002-09-01 23:40:07-07  ball
 * Combined Position class with Tile class.
 *
 * Revision 1.4  2002-09-01 22:01:38-07  ball
 * Consistently specify "RobotMap".
 *
 * Revision 1.3  2002-09-01 15:08:40-07  ball
 * Made setting of command arguments consistent for all subclasses
 * of Command.
 * Added an initialization method to Strategy classes.
 * Player will submit a no-op move if the Strategy implementation
 * does not return a command.
 *
 * Revision 1.2  2002-09-01 08:38:12-07  ball
 * Properly handle direction and package arguments to server commands.
 * Move Strategy class implementations to their own package.
 * Make board two rows higher and two columns wider than the server
 * specifies.  No longer translate one-based server coordinates to
 * zero-based co-ordinates.  Make the unspecified tiles on the edges wall
 * tiles.  This removes all special handling logic because all edge tiles
 * are guaranteed to be wall tiles (and, therefore, impassable).
 *
 * Revision 1.1  2002-08-31 17:46:24-07  ball
 * Initial revision
 *
 */
