/*
 * $Id: PlanA.java,v 1.2 2002-09-02 10:12:05-07 ball Exp $
 *
 * Copyright 2002 Allen D. Ball.  All rights reserved.
 */
package player.strategy;

import player.*;
import world.*;

import java.util.*;

/**
 * This class implements the Plan A strategy.
 *
 * @author <a href="mailto:ball@iprotium.com">Allen D. Ball</a>
 * @version $Revision: 1.2 $
 */
public class PlanA extends Strategy {
    static private Command noop = new NoopCommand();

    private Path path = null;

    /**
     * Creates a Plan A strategy.
     */
    public PlanA() { super(); }

    /**
     * Method to initialize the Plan A strategy.
     *
     * @param   player          The player on whose behalf this
     *                          strategy will operate.
     */
    public void initialize(Player player) {
        super.initialize(player);

        Board board = player.getWorld().getBoard();

        travelLog = new boolean[board.getRowCount()][board.getColumnCount()];
    }

    private boolean[][] travelLog = null;

    private void chalk(Tile tile) { chalk(tile.getRow(), tile.getColumn()); }

    private void chalk(int row, int column) { travelLog[row][column] = true; }

    private boolean haveVisited(Tile tile) {
        return haveVisited(tile.getRow(), tile.getColumn());
    }

    private boolean haveVisited(int row, int column) {
        return travelLog[row][column];
    }

    /**
     * Method to calculate the player's robot next command using
     * "Plan A."
     *
     * @return                  The next command to send to the player's
     *                          robot.
     */
    public Command nextCommand() {
        World world = player.getWorld();
        Board board = world.getBoard();
        PlayerRobot robot = player.getPlayerRobot();
        Pkg[] robotPkgs = robot.getSack().getPkgs();
        Tile tile = robot.getTile();

        Command command = null;
        int bid = 1;

        chalk(tile);

        /* Is this tile the destination of any packages we are carrying?
         */
        if (command == null && robotPkgs.length > 0) {
            command = new DropCommand();

            for (int i = 0; i < robotPkgs.length; i += 1) {
                if (robotPkgs[i].getDestination() == tile) {
                    command.pkgAdd(robotPkgs[i]);
                }
            }

            /* No packages to drop.
             */
            if (command.getArguments().equals("")) {
                command = null;
            }
        }

        /* Does this tile have any packages we want to pick up?
         */
        if (command == null && tile.getSack().getCount() > 0) {
            command = new PickCommand();

            int available = robot.getCapacity() - robot.getLoad();

            Pkg[] pkgs = tile.getSack().getPkgs();
            for (int i = 0; i < pkgs.length; i += 1) {
                if (pkgs[i].getWeight() <= available) {
                    command.pkgAdd(pkgs[i]);
                    available -= pkgs[i].getWeight();
                }
            }

            /* No packages to pick up.
             */
            if (command.getArguments().equals("")) {
                command = null;
            }
        }

        /* Choose a path.
         */
        path = null;

        /* If we have a package, find the closest destination we have
         * package(s) for.
         */
        if (command == null && path == null && robotPkgs.length > 0) {
            int closest = Integer.MAX_VALUE;
            int distance = 0;

            for (int i = 0; i < robotPkgs.length; i += 1) {
                distance =
                    tile.getPathTo(robotPkgs[i].getDestination())
                    .getDistance();
                if (distance < closest) {
                    path = tile.getPathTo(robotPkgs[i].getDestination());
                    closest = distance;
                }
            }
        }

        /* If we know where some packages are, go get them.  Only go
         * after packages that we know the weight (either we have
         * visited the base or we held them once).
         */
        if (command == null && path == null) {
            int foundPkgWeight = 0;
            int tilePkgWeight = 0;

            for (int i = 0; i < board.getRowCount(); i += 1) {
                for (int j = 0; j < board.getColumnCount(); j += 1) {
                    tilePkgWeight = board.getTile(i, j).getSack().getWeight();
                    if (tilePkgWeight > foundPkgWeight) {
                        path = tile.getPathTo(board.getTile(i, j));
                        foundPkgWeight = tilePkgWeight;
                    }
                }
            }
        }

        /* If we don't know where some packages are, head for a base.
         * If we have already been to a base, the previous sub-rule
         * would have kicked in.  Therefore, a base we have been to (and
         * do know the inventory) is *not* a candidate.  Go to the
         * closest that is a candidate.
         */
        if (command == null && path == null) {
            int closest = Integer.MAX_VALUE;
            int distance = 0;

            for (int i = 0; i < board.getRowCount(); i += 1) {
                for (int j = 0; j < board.getColumnCount(); j += 1) {
                    if (board.getTile(i, j) instanceof BaseTile
                        && (! haveVisited(i, j))) {
                        distance =
                            tile.getPathTo(board.getTile(i, j)).getDistance();
                        if (distance < closest) {
                            path = tile.getPathTo(board.getTile(i, j));
                            closest = distance;
                        }
                    }
                }
            }
        }

        /* Again, if we know where some packages are, go get them.
         * this time we are left with packages that we do not know
         * their weight.
         */
        if (command == null && path == null) {
            int foundPkgCount = 0;
            int tilePkgCount = 0;

            for (int i = 0; i < board.getRowCount(); i += 1) {
                for (int j = 0; j < board.getColumnCount(); j += 1) {
                    tilePkgCount = board.getTile(i, j).getSack().getCount();
                    if (tilePkgCount > foundPkgCount) {
                        path = tile.getPathTo(board.getTile(i, j));
                        foundPkgCount = tilePkgCount;
                    }
                }
            }
        }

        /* No more packages on the board.  Go after the opponent with
         * the most packages (hoping we may encounter some "droppings").
         */
        if (command == null && path == null) {
            Robot[] robots =
                (Robot[]) world.getRobotMap().values().toArray(new Robot[0]);
            int foundPkgCount = 0;
            int robotPkgCount = 0;

            for (int i = 0; i < robots.length; i += 1) {
                if (robots[i] instanceof OpponentRobot) {
                    robotPkgCount = robots[i].getSack().getCount();
                    if (robotPkgCount > foundPkgCount) {
                        path = tile.getPathTo(robots[i].getTile());
                        foundPkgCount = robotPkgCount;
                    }
                }
            }
        }

        /* If we settled on a destination, create the move command.
         */
        if (command == null) {
            if (path != null && path.getDistance() > 0) {
                command = new MoveCommand();
                command.setDirection(path.getSteps()[0]);
            }
        }

        /* Now that we have settled on the command, examine defensive
         * options.
         */
        TreeSet opponentPositions = world.getOpponentPositions();
        TreeSet opponentMoves = world.getOpponentMoves();

        if (command != null) {
            if (command instanceof DropCommand) {
                /*
                 * Try to hold our ground if we want to drop our packags
                 * by bidding higher.
                 */
                if (opponentMoves.contains(tile)) {
                    if (robot.getMoney() > 1) {
                        bid = 2;
                    }
                }
            }

            if (command instanceof MoveCommand) {
                /*
                 * Try to steer around everyone else.
                 */
                if (opponentMoves.contains(path.getTiles()[0])) {
                }
            }
        }

        /* Set our bid.
         */
        if (command != null) {
            command.setBid(bid);
        }

        /* Just in case the above logic fails.
         */
        if (command == null) {
            command = noop;
        }

        return command;
    }
}
/*
 * $Log: PlanA.java,v $
 * Revision 1.2  2002-09-02 10:12:05-07  ball
 * Checked in strategy work-in-progress to make a contest submission.
 *
 * Revision 1.1  2002-09-01 23:42:52-07  ball
 * Initial revision
 *
 */
