/*
 * $Id: Tile.java,v 1.7 2002-09-02 10:13:55-07 ball Exp $
 *
 * Copyright 2002 Allen D. Ball.  All rights reserved.
 */
package world;

import java.util.*;

/**
 * Abstract class that implements board tiles for the ICFP-2002
 * programming contest entry.
 *
 * @author <a href="mailto:ball@iprotium.com">Allen D. Ball</a>
 * @version $Revision: 1.7 $
 */
public abstract class Tile implements Comparable, Direction, Locality {
    private Board board;
    private int row = -1;       /* Y */
    private int column = -1;    /* X */
    private Sack sack = new Sack();

    /**
     * Creates a tile.
     *
     * @param   board           The playing board.
     * @param   row             The row position of the tile.
     * @param   column          The column position of the tile.
     *
     * @see Position
     */
    protected Tile(Board board, int row, int column) {
        this.board = board;
        this.row = row;
        this.column = column;
    }

    private Tile() { }

    /**
     * Method to get the containing board.
     *
     * @return                  The containing board.
     */
    public Board getBoard() { return board; }

    /**
     * Method to get the tile row.
     *
     * @return                  The tile row.
     */
    public int getRow() { return row; }

    /**
     * Method to get the tile column.
     *
     * @return                  The tile column.
     */
    public int getColumn() { return column; }

    /**
     * Method to get the tile's sack.
     *
     * @return                  The tile's sack.
     */
    public synchronized Sack getSack() { return sack; }

    /**
     * Method to return tile.  Used to implement Locality.
     *
     * @return                  The tile.
     */
    public synchronized Tile getTile() { return this; }

    /**
     * Method to get the tile's neighbor.
     *
     * @return                  The neighboring tile.
     *
     * @see Direction
     */
    public Tile getNeighbor(int direction) {
        Tile neighbor = null;

        int r = row;
        int c = column;

        switch (direction) {
        case NORTH:
            r += 1;
            break;

        case SOUTH:
            r -= 1;
            break;

        case EAST:
            c += 1;
            break;

        case WEST:
            c -= 1;
            break;
        }

        if (0 <= r && r < board.getRowCount()
            && 0 <= c && c < board.getColumnCount()) {
            neighbor = board.getTile(r, c);
        }

        return neighbor;
    }

    /**
     * Method to determine if the tile is passable.
     *
     * @return                  True if a robot may occupy the space;
     *                          false otherwise.
     */
    public abstract boolean isPassable();

    /**
     * Method to get the set of possible passages from this tile.
     *
     * @return                  A sorted set of tiles.
     *
     * @see Tile
     */
    public TreeSet getPassages() {
        TreeSet set = new TreeSet();

        if (getNeighbor(NORTH).isPassable()) {
            set.add(getNeighbor(NORTH));
        }

        if (getNeighbor(SOUTH).isPassable()) {
            set.add(getNeighbor(Direction.SOUTH));
        }

        if (getNeighbor(EAST).isPassable()) {
            set.add(getNeighbor(Direction.EAST));
        }

        if (getNeighbor(WEST).isPassable()) {
            set.add(getNeighbor(WEST));
        }

        return set;
    }

    /**
     * Method to determine if the tile is lethal.
     *
     * @return                  True if a robot will be killed if it
     *                          enters the tile; false otherwise.
     */
    public abstract boolean isLethal();

    /**
     * Method to return the path from this tile to the end tile.
     *
     * @param   end             The end tile of the path.
     *
     * @return                  The path from start to end.
     */
    public Path getPathTo(Tile end) { return board.getPath(this, end); }

    /**
     * Compares this object with the specified object for order.
     * Returns a negative integer, zero, or a positive integer as this
     * object is less than, equal to, or greater than the specified
     * object.
     *
     * @param   object          The object to be compared.
     *
     * @see java.lang.Comparable
     */
    public synchronized int compareTo(Object object) {
        return getOrdinal() - ((Tile) object).getOrdinal();
    }

    /**
     * Method to get an ordinal ordering for this tile with the other
     * tiles in its board.
     */
    public int getOrdinal() { return (board.getRowCount() * row) + column; }
}
/*
 * $Log: Tile.java,v $
 * Revision 1.7  2002-09-02 10:13:55-07  ball
 * Better use of sets.
 * Added trail to robots.
 * Optimized path calculation code (although there are still obvious
 * optimizations to be made).
 *
 * Revision 1.6  2002-09-02 03:22:25-07  ball
 * Store calculated paths for re-use.
 *
 * Revision 1.5  2002-09-01 23:38:55-07  ball
 * Combined Position class with Tile class.  Removed Position class.
 *
 * Revision 1.4  2002-09-01 22:08:34-07  ball
 * Removed extra GUI support from IdMap and subclasses -- The GUI used in
 * development will be updated at the end of every turn.
 * The Position class implements Comparable.  Also added a toString()
 * method suitable for displaying the coordinates of the position.
 * Added a method to the Tile class for calculating the path between tiles.
 * Marked a water tile as "impassable."
 *
 * Revision 1.3  2002-09-01 15:10:58-07  ball
 * Tiles and Robots both use Sacks to hold packages.
 * Added standard Directions.  Added methods to Tile and Position
 * to take advantage of Directions.
 * GUI is no longer World-specific.
 *
 * Revision 1.2  2002-09-01 08:47:52-07  ball
 * Added position information to tiles.
 * Moved most diagnostic GUI code to World class.
 * Track player robot's actual and opponent robot's observed scores and
 * loads.  Track opponent robot's observed capacity.  Added methods to
 * manage, drop, and pick packages.
 * Add load and position columns to robot table.
 * Track game turn number.
 * Display robots on the world map.
 *
 * Revision 1.1  2002-08-31 17:52:00-07  ball
 * Initial revision
 *
 */
