/*
 * $Id: World.java,v 1.6 2002-09-02 10:13:55-07 ball Exp $
 *
 * Copyright 2002 Allen D. Ball.  All rights reserved.
 */
package world;

import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;

/**
 * Class that implements the World representation for the simulation in
 * the ICFP-2002 programming contest.
 *
 * @author <a href="mailto:ball@iprotium.com">Allen D. Ball</a>
 * @version $Revision: 1.6 $
 */
public class World {
    private Board board = null;;
    private int turn = 0;
    private RobotMap robotMap = new RobotMap();
    private AbstractTableModel tableModel = null;

    /**
     * Creates the World.
     */
    public World() { }

    /**
     * Method to get the current turn.
     *
     * @return                  The current turn.
     */
    public int getTurn() { return turn; }

    /**
     * Method to move to the next turn.
     */
    public void turn() { this.turn += 1; }

    /**
     * Method to get world's playing board.
     *
     * @return                  The playing board.
     */
    public Board getBoard() { return board; }

    /**
     * Method to set world's playing board.
     *
     * @param   board           The playing board.
     */
    public void setBoard(Board board) { this.board = board; }

    /**
     * Method to get robot map.
     *
     * @return                  The map of robots on this board.
     */
    public RobotMap getRobotMap() { return robotMap; }

    /**
     * Method to get the set of opponent robot positions.
     *
     * @return                  A sorted set of tiles.
     *
     * @see Tile
     */
    public TreeSet getOpponentPositions() {
        TreeSet set = new TreeSet();
        Robot[] robots = (Robot[]) robotMap.values().toArray(new Robot[0]);

        for (int i = 0; i < robots.length; i += 1) {
            if (robots[i] instanceof OpponentRobot) {
                set.add(robots[i].getTile());
            }
        }

        return set;
    }

    /**
     * Method to get the set of possible opponent robot moves.
     *
     * @return                  A sorted set of tiles.
     *
     * @see Tile
     */
    public TreeSet getOpponentMoves() {
        Tile[] tiles = (Tile[]) getOpponentPositions().toArray(new Tile[0]);
        TreeSet set = new TreeSet();

        for (int i = 0; i < tiles.length; i += 1) {
            set.addAll(tiles[i].getPassages());
        }

        return set;
    }

    /**
     * Returns a table model for displaying the world map.
     */
    public TableModel getTableModel() {
        if (tableModel == null) {
            tableModel = new WorldTableModel();
        }

        return tableModel;
    }

    private class WorldTableModel extends AbstractTableModel {
        public int getColumnCount() { return board.getColumnCount(); }

        public String getColumnName(int c) { return Integer.toString(c); }

        public int getRowCount() { return board.getRowCount(); }

        public Object getValueAt(int r, int c) {
            Tile tile = (Tile) board.getTile(getRowCount() - (r + 1), c);

            String pkgs = "";
            if (tile.getSack().getCount() > 0) {
                pkgs =
                    "(" + tile.getSack().getCount() + ")"
                    + tile.getSack().getWeight();
            }

            String tileType = "";
            if (tile instanceof BaseTile) {
                tileType = "@";
            } else if (tile instanceof PlainTile) {
                tileType = " ";
            } else if (tile instanceof WallTile) {
                tileType = "#";
            } else if (tile instanceof WaterTile) {
                tileType = "~";
            }

            String robotNames = "";
            Robot[] robots = (Robot[]) robotMap.values().toArray(new Robot[0]);
            for (int i = 0; i < robots.length; i += 1) {
                if (robots[i].getTile() == tile) {
                    if (robotNames != "") {
                        robotNames += ",";
                    }

                    robotNames += "R" + robots[i].getId();
                }
            }

            return pkgs + tileType + robotNames;
        }
    }
}
/*
 * $Log: World.java,v $
 * Revision 1.6  2002-09-02 10:13:55-07  ball
 * Better use of sets.
 * Added trail to robots.
 * Optimized path calculation code (although there are still obvious
 * optimizations to be made).
 *
 * Revision 1.5  2002-09-01 23:38:55-07  ball
 * Combined Position class with Tile class.  Removed Position class.
 *
 * Revision 1.4  2002-09-01 22:08:34-07  ball
 * Removed extra GUI support from IdMap and subclasses -- The GUI used in
 * development will be updated at the end of every turn.
 * The Position class implements Comparable.  Also added a toString()
 * method suitable for displaying the coordinates of the position.
 * Added a method to the Tile class for calculating the path between tiles.
 * Marked a water tile as "impassable."
 *
 * Revision 1.3  2002-09-01 15:10:58-07  ball
 * Tiles and Robots both use Sacks to hold packages.
 * Added standard Directions.  Added methods to Tile and Position
 * to take advantage of Directions.
 * GUI is no longer World-specific.
 *
 * Revision 1.2  2002-09-01 08:47:52-07  ball
 * Added position information to tiles.
 * Moved most diagnostic GUI code to World class.
 * Track player robot's actual and opponent robot's observed scores and
 * loads.  Track opponent robot's observed capacity.  Added methods to
 * manage, drop, and pick packages.
 * Add load and position columns to robot table.
 * Track game turn number.
 * Display robots on the world map.
 *
 * Revision 1.1  2002-08-31 17:52:00-07  ball
 * Initial revision
 *
 */
